﻿using System;
using System.Threading.Tasks;
using Outlook = Microsoft.Office.Interop.Outlook;
using Microsoft.Office.Tools;



using POCAddIn.Ribbons;

using POCAddIn;
using POCAddIn.Extensions;

namespace POCAddIn.Wrappers
{
    internal abstract class GenericOutlookContainer<T> : IDisposable where T : class
    {
        private T _wrappedObject;
        private CustomTaskPane taskPane;
        Outlook.Explorer _explorer;
        Outlook.Inspector _inspector;
        protected IRibbonContainer _ribbonContainer;
        internal event EventHandler<CustomTaskPane> Close;


        internal GenericOutlookContainer(T wrappedObject)
        {
            try
            {
                Logger = new InternalLogManager();
                //Bootstrap(container);
                _wrappedObject = wrappedObject;

                string title;
                _inspector = _wrappedObject as Outlook.Inspector;
                if (_inspector != null)
                {
                    title = "Pain in inspector";
                    ((Outlook.InspectorEvents_10_Event)_inspector).Close += GenericOutlookContainer_Close;
                    ((Outlook.InspectorEvents_10_Event)_inspector).Activate += GenericOutlookContainer_Activate;
                    ((Outlook.InspectorEvents_10_Event)_inspector).Deactivate += GenericOutlookContainer_Deactivate;

                }

                else
                {
                    _explorer = _wrappedObject as Outlook.Explorer;
                    if (_explorer != null)
                    {
                        title = "Pain in explorer";

                        ((Outlook.ExplorerEvents_10_Event)_explorer).Close += GenericOutlookContainer_Close;
                        ((Outlook.ExplorerEvents_10_Event)_explorer).Activate += GenericOutlookContainer_Activate;
                        ((Outlook.ExplorerEvents_10_Event)_explorer).Deactivate += GenericOutlookContainer_Deactivate;
                        _explorer.SelectionChange += _explorer_SelectionChange;

                    }
                    else
                    {
                        //   throw new ArgumentException("WTF?");
                        return;
                    }
                }

                //taskPane = Globals.ThisAddIn.CustomTaskPanes.Add(new WpfPanelHost { Child = _taskPaneContentHost.AsUIElement() }, title, _wrappedObject);
                //taskPane.VisibleChanged += taskPane_VisibleChanged;

            }
            catch (Exception e)
            {

                if (Logger != null)
                    Logger.Error(this, e, string.Empty);
            }
        }

        protected internal abstract void GenericOutlookContainer_Deactivate();

        protected internal abstract void GenericOutlookContainer_Activate();

        //protected ILifetimeScope LifeTimeScope { get; private set; }

        //protected internal abstract OutLookWrapperBase SelectedOutlookItem
        //{
        //    get;
        //    protected set;
        //}
        internal abstract void _explorer_SelectionChange();

        internal virtual async Task Initialise()
        {

            CustomTaskPane.Visible = !CustomTaskPane.Visible;
        }
        protected abstract void RefreshRibbon();


        protected virtual void GenericOutlookContainer_Close()
        {

            try
            {
                GC.Collect();
                GC.WaitForPendingFinalizers();
                GC.Collect();
                GC.WaitForPendingFinalizers(); //has to be caleld twice
                OnClose();
                if (taskPane != null)
                {

                    taskPane.VisibleChanged -= taskPane_VisibleChanged;
                    taskPane.Dispose();
                    taskPane = null;
                }


                if (_explorer != null)
                {
                    ((Outlook.ExplorerEvents_Event)_explorer).Close -=
                        new Outlook.ExplorerEvents_CloseEventHandler(
                        GenericOutlookContainer_Close);
                    _explorer.SelectionChange -= _explorer_SelectionChange;
                    _explorer.FinalReleaseComObject();
                    _explorer = null;

                }

                if (_inspector != null)
                {
                    ((Outlook.InspectorEvents_Event)_inspector).Close -= GenericOutlookContainer_Close;
                    ((Outlook.InspectorEvents_Event)_inspector).Activate -= GenericOutlookContainer_Activate;
                    ((Outlook.InspectorEvents_Event)_inspector).Deactivate -= GenericOutlookContainer_Deactivate;
                    _inspector.FinalReleaseComObject();
                    _inspector = null;
                }

                if (_wrappedObject != null)
                {
                    _wrappedObject.FinalReleaseComObject();
                    _wrappedObject = null;
                }
                _ribbonContainer = null;
            }
            catch (Exception e)
            {

                //TODO: Log exception

            }
            finally
            {
                //LifeTimeScope.Dispose();
                //LifeTimeScope = null;
                GC.Collect();
                GC.WaitForPendingFinalizers();
                GC.Collect();
                GC.WaitForPendingFinalizers(); //has to be caleld twice
            }



        }

        void OnClose()
        {
            var close = this.Close;
            if (close != null)
                close(this, CustomTaskPane);
        }

        protected virtual void taskPane_VisibleChanged(object sender, EventArgs e)
        {
            //do some stuff here
        }

        internal T OutlookObject
        {
            get { return _wrappedObject; }
        }
        internal CustomTaskPane CustomTaskPane
        {
            get { return taskPane; }
        }

        protected ILogManager Logger { get; private set; }
        public virtual void Dispose()
        {
            GenericOutlookContainer_Close();
        }
    }



}
