﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;
using Outlook = Microsoft.Office.Interop.Outlook;
using Office = Microsoft.Office.Core;
using Microsoft.Office.Tools.Ribbon;
using Microsoft.Office.Tools;
using System.Diagnostics;
using POCAddIn.Wrappers;
using POCAddIn.Ribbons;



namespace POCAddIn
{


    public enum RibbonState
    {
        Create,
        Display
    }


    public partial class ThisAddIn
    {
        Outlook.Explorer currentExplorer = null;
        IRibbonContainer _ribbon;
        private Dictionary<Outlook.Inspector, GenericOutlookContainer<Outlook.Inspector>> inspectorWrappersValue = new Dictionary<Outlook.Inspector, GenericOutlookContainer<Outlook.Inspector>>();
        private Outlook.Inspectors inspectors;
        internal static Office.IRibbonUI RibbonUI;
        Outlook.Application _application;
        Outlook.Explorers m_Explorers;
        private Dictionary<Outlook.Explorer, GenericOutlookContainer<Outlook.Explorer>> _explorerWrapers = new Dictionary<Outlook.Explorer, GenericOutlookContainer<Outlook.Explorer>>();
        //private ILifetimeScope _lifetimeScope;

        private void NewInspector_Launched(Outlook.Inspector inspector)
        {

            var existingInspector = WindowAlreadyWrapped<Outlook.Inspector>(inspector);
            if (existingInspector == false)
            {
                GenericOutlookContainer<Outlook.Inspector> wrapper = new OutlookInspectorWrapper(inspector);
                wrapper.Close += InspectorContainer_Closed;
                inspectorWrappersValue.Add(inspector, wrapper);
            }

        }

        //internal Autofac.IContainer DependencyResolver { get { return Bootstrapper.Container; } }
        private void NewExplorer_Launched(Outlook.Explorer explorer)
        {
            try
            {

                var existingWindow = WindowAlreadyWrapped<Outlook.Explorer>(explorer);

                if (existingWindow == false)
                {

                    OutlookExplorerWrapper window = new OutlookExplorerWrapper(explorer);
                    _explorerWrapers.Add(explorer, window);
                    window.Close += InspectorContainer_Closed;

                    //_outlookExplorersList.Add(window);
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
            }
        }

        void InspectorContainer_Closed(object sender, CustomTaskPane e)
        {
            try
            {
                this.CustomTaskPanes.Remove(e);

                OutlookInspectorWrapper wrapper = sender as OutlookInspectorWrapper;
                if (wrapper != null)
                {
                    inspectorWrappersValue.Remove(wrapper.OutlookObject);

                }
                else
                {
                    OutlookExplorerWrapper wrap = sender as OutlookExplorerWrapper;
                    if (wrap != null)
                    {
                        _explorerWrapers.Remove(wrap.OutlookObject);
                    }
                }
            }
            catch (Exception ex)
            {

                //TfsLogManager.Instance.Error(this, ex, string.Empty);
            }

        }

        internal Dictionary<Outlook.Inspector, GenericOutlookContainer<Outlook.Inspector>> InspectorWrappers
        {
            get
            {
                return inspectorWrappersValue;
            }
        }


        internal Dictionary<Outlook.Explorer, GenericOutlookContainer<Outlook.Explorer>> ExplorerWrappers
        {
            get
            {
                return _explorerWrapers;
            }
        }


        private bool WindowAlreadyWrapped<T>(T window) where T : class
        {
            if (typeof(T) == typeof(Outlook.Explorer))
            {
                foreach (OutlookExplorerWrapper Explorer in _explorerWrapers.Values)
                {
                    if (Explorer.OutlookObject == window)
                    {
                        return true;
                    }
                }
            }
            else
                foreach (OutlookInspectorWrapper inspector in InspectorWrappers.Values)
                {
                    if (inspector.OutlookObject == window)
                    {
                        return true;
                    }
                }
            return false;
        }

        
        private void ThisAddIn_Startup(object sender, System.EventArgs e)
        {
            try
            {
                _application = this.Application;
                currentExplorer = this.Application.ActiveExplorer();
                m_Explorers = _application.Explorers;
                inspectors = this.Application.Inspectors;



                inspectors.NewInspector +=
                    new Outlook.InspectorsEvents_NewInspectorEventHandler(
                    NewInspector_Launched);

                m_Explorers.NewExplorer +=
                   new Outlook.ExplorersEvents_NewExplorerEventHandler(
                       NewExplorer_Launched);



                OutlookExplorerWrapper window = new OutlookExplorerWrapper(currentExplorer);
                window.Close += InspectorContainer_Closed;

                _explorerWrapers.Add(currentExplorer, window);



                foreach (Outlook.Inspector inspector in inspectors)
                {
                    NewInspector_Launched(inspector);
                }


            }
            catch (Exception ex)
            {

                
            }


        }

        private void ThisAddIn_Shutdown(object sender, System.EventArgs e)
        {
            inspectors.NewInspector -= NewInspector_Launched;
            inspectors = null;
            m_Explorers.NewExplorer -= NewExplorer_Launched;
            m_Explorers = null;

            //foreach (var x in inspectorWrappersValue)
            //{
            //    x.Value.Dispose();
            //}
            inspectorWrappersValue.Clear();
            inspectorWrappersValue = null;

        }

    

        protected override Office.IRibbonExtensibility CreateRibbonExtensibilityObject()
        {
            try
            {


                
                _ribbon = new Ribbon();
                return _ribbon;
            }
            catch (Exception e)
            {

                
            }
            
            return null;
        }



        #region VSTO generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InternalStartup()
        {
            this.Startup += new System.EventHandler(ThisAddIn_Startup);
            this.Shutdown += new System.EventHandler(ThisAddIn_Shutdown);
        }

        #endregion
    }
}
